* Example QSPIL to call lfax
* L.K.Wareham June 1994
*
* Based on
* QSPIL for US Robotics Sportster 14400 with FAX
* V0.06, June 1994 L.K.Wareham
* based on
* Generic QSPIL driver for Hayes-compatible modems
* by Jan B v0.99b, August 4, 1990

* Notes: This version is configured for a ser2 locked baud rate
* but the code for matching the modem rate exists but is untested.
* Unsupported V32 rates are set for the next highest supported
* rate. If you want to be able to connect at 12000 or 14400 your
* platform must support 19200 which means HERMES for QL owners.
* Also this QSPIL is configured for full hardware handshake,
* If you want to use the extended driver CD facility
* you should use ser1 or disable hardware handshaking for
* ser2. This may require you to operate with the serial port
* baud rate unlocked.

* The extended driver baud rate call has
* been removed to stop any collision with Hermes.
* Use the extended Hermes calls to lock ser1's rate in
* your boot.

* The MNP5 data compression is inhibited as this is not
* efficient with compressed files. V42bis compression can
* be auto-negotiated.

* Connect speeds of 7200, 12000 and 14400 are reported as
* 9600 to QBOX and MAIL software for compatibility. The real
* speed and any error correction/data compression is
* reported to the log directly.
* This  means you get two connect speeds, the real one and
* 9600.

* This modem DOES support V23 in the autospeed sequence if
* enabled, also enabled is the Euro V21 answer sequence.
* Note V23 connect returns CONNECT 1200 not CONNECT 1275
* so it is not possible to know if you'r connected at
* 1200/75 or 1200, always returns a 1200 result.

* The first initialise instruction sets the modem to data mode
* in case FAX software has been run.

* The carrier detect time has been reduced.

* Tone dialing is defaulted.

* Note also that the binary MUST be linked.

* Macro for calling QBOX's PRLOG routine

STRING$  MACRO    A
[.LAB]   DC.W     .E.[.L]-*-2
         DC.B     [A]
.E.[.L]  EQU      *
         ENDM

PRLOG    MACRO    A
[.LAB]   LEA      .S.[.L],A1
         JSR      (A4)
         SECTION  MSG
.S.[.L]  STRING$  {[A]}
         SECTION  CODE
         ENDM

PRINT    MACRO    A
[.LAB]   LEA      .S.[.L],A1
         MOVE.W   UT_MTEXT,A2
         JSR      (A2)
         SECTION  MSG
.S.[.L]  STRING$  {[A]}
         SECTION  CODE
         ENDM

CR       EQU      $0D
LF       EQU      $0A

ERR.NC   EQU      -1
ERR.NO   EQU      -6
ERR.IU   EQU      -9
ERR.EF   EQU      -10
ERR.BP   EQU      -15
ERR.NI   EQU      -19

* QDOS calls

MT.SUSJB EQU      8
MT.BAUD  EQU      $12

IO.OPEN  EQU      1
IO.CLOSE EQU      2

IO.PEND  EQU      0
IO.FBYTE EQU      1
IO.FLINE EQU      2
IO.SBYTE EQU      5
IO.SSTRG EQU      7
FS.FLUSH EQU      $41

UT_MTEXT EQU      $D0

* Extra TRAP #3 keys for extended SER2 driver

IO.DUMP  EQU      $50              Dump unsent data
IO.PARIT EQU      $51              Set parity
IO.BAUD  EQU      $52              Set baudrate

* D0 keys for QSPIL calls

SP.INIT  EQU      0                 Initialise driver
SP.WTCAL EQU      1                 Wait for call
SP.DSCON EQU      2                 Disconnect
SP.CLOSE EQU      3                 Closedown
SP.PEND  EQU      8                 Test pending input
SP.FBYTE EQU      9                 Fetch byte
SP.SBYTE EQU      $0A               Send byte
SP.SSTRG EQU      $0B               Send string
SP.FLUSH EQU      $0C               Flush output buffer
SP.DUMP  EQU      $0D               Dump unsent data
SP.PARIT EQU      $0E               Set parity
SP.MAXOP EQU      $0E               Highest operation code

* Return keys for SP.WTCAL

CON_300  EQU      0                 Connection at 300/300 bps
CON_1275 EQU      1                 Connection at 1200/75 bps (main channel)
                                    ;(can never return this)
CON_1200 EQU      2                 Connection at 1200/1200 bps
CON_2400 EQU      3                 Connection at 2400 bps
CON_4800 EQU      4
CON_9600 EQU      5
CON_7200 EQU      6
CON12000  EQU     7
CON14400  EQU     8
CON_FAX   EQU     9                 <fax connect>
CON_NC    EQU    10                 <data con fail due to fax>

FAXCALL   EQU     1                 <return in d0 when lfax called>
AMODE     EQU     1                 <flag for connect in d7.b answer mode>
CMODE     EQU     2                 <for other modes>

         XREF    confax,dcffax       ;<fax interface module entry points>

         SECTION  CODE

         JMP      START

* Configuration data

CFGVER   DC.W     100
BAUD     DC.W     19200
LOCKED   DC.W     1
DELAY    DC.W     1
ANSWER   DC.W     1
PORT     STRING$  'SER2H'
         DS.B     32-(*-PORT)
OKMSG    STRING$  'OK'
         DS.B     32-(*-OKMSG)
NOCARMSG STRING$  {'NO CARRIER'}
         DS.B     32-(*-NOCARMSG)
BUSYMSG  STRING$  'BUSY'
         DS.B     32-(*-BUSYMSG)
RINGMSG  STRING$  'RING|'
         DS.B     32-(*-RINGMSG)
CONN300  STRING$  'CONNECT|'
         DS.B     32-(*-CONN300)
CONN1275 STRING$  {'CONNECT 1275'}              Sportster can't return this
         DS.B     32-(*-CONN1275)
CONN1200 STRING$  {'CONNECT 1200'}
         DS.B     32-(*-CONN1200)
CONN2400 STRING$  {'CONNECT 2400'}
         DS.B     32-(*-CONN2400)
CONN4800 STRING$  {'CONNECT 4800'}
         DS.B     32-(*-CONN4800)
CONN9600 STRING$  {'CONNECT 9600'}
         DS.B     32-(*-CONN9600)
CONN7200 STRING$  {'CONNECT 7200'}
         DS.B     32-(*-CONN7200)
COS12000 STRING$  {'CONNECT 12000'}
         DS.B     32-(*-COS12000)
COS14400 STRING$  {'CONNECT 14400'}
         DS.B     32-(*-COS14400)

CONNFAX  STRING$  {'CONNECT FAX'}       ;<the fax auto detect message>
         DS.B     32-(*-CONNFAX)        ;<may be different for other modems>

CONNFAIL STRING$  {'NO CARRIER'}        ;<message when fails to connect to data>
         DS.B     32-(*-CONNFAIL)       ;<due to fax may be different for other modems>

INIT1    STRING$  'AT+FCLASS=0|'
         DS.B     50-(*-INIT1)
INIT2    STRING$  'ATB0E0F1H0M2Q0V1X4S0=0S27=1S34=14|'
         DS.B     50-(*-INIT2)
INIT3    STRING$  'AT&A3&N0&B1&K3&M4&H1&R2S7=30|'
         DS.B     50-(*-INIT3)
ASWCMD   STRING$  '~ATS7=15A|'          ;<note the reduced carrier detect time>
                                        ;<in answer mode>
                                        ;<not needed if modem supports auto>
                                        ;<detect of data/fax mode>
         DS.B     50-(*-ASWCMD)
HANGUP   STRING$  '~~~+++~~~ATH0|'
         DS.B     50-(*-HANGUP)
OFFLINE  STRING$  'ATH1|'
         DS.B     50-(*-OFFLINE)
DIAL300  STRING$  'ATDT'
         DS.B     50-(*-DIAL300)
DIAL1275 STRING$  'ATDT'
         DS.B     50-(*-DIAL1275)
DIAL1200 STRING$  'ATDT'
         DS.B     50-(*-DIAL1200)
DIAL2400 STRING$  'ATDT'
         DS.B     50-(*-DIAL2400)
DIAL4800 STRING$  'ATDT'
         DS.B     50-(*-DIAL4800)
DIAL9600 STRING$  'ATDT'
         DS.B     50-(*-DIAL9600)
DSUFFIX  STRING$  'S7=30|'              ;<make sure originate has long enough>
                                        ;<Carrier Detect Time>
         DS.B     50-(*-DSUFFIX)

START    CMPI.W   #SP.MAXOP,D0      Valid function call?
         BHI.S    ERR_NI            Jump if not implemented yet
         ADD.W    D0,D0             Calculate table offset
         MOVE.W   OPTBL(PC,D0.W),D0
         JMP      START(PC,D0.W)    Jump to handling routine
ERR_NI   MOVEQ    #ERR.NI,D0        "Not implemented"
         RTS
OPTBL    DC.W     SP_INIT-START
         DC.W     SP_WTCAL-START
         DC.W     SP_DSCON-START
         DC.W     SP_CLOSE-START
         DC.W     SP_FBAUD-START
         DC.W     SP_DIAL-START
         DC.W     ERR_NI-START
         DC.W     ERR_NI-START
         DC.W     SP_PEND-START
         DC.W     SP_FBYTE-START
         DC.W     SP_SBYTE-START
         DC.W     SP_SSTRG-START
         DC.W     SP_FLUSH-START
         DC.W     SP_DUMP-START
         DC.W     SP_PARIT-START

* Initialise driver
                                    
SP_INIT  BSR      OPENCHAN          Is it possible to open a channel?
         BNE.S    INIT_END          No, exit
         MOVEQ    #IO.DUMP,D0
         MOVEQ    #-1,D3
         TRAP     #3                Test for extended SER2-driver
         TST.L    D0
         BEQ.S    IN_INIT
         MOVE.L   A0,-(A7)
         SUBA.L   A0,A0
         PRINT    {'WARNING: QBOX needs extended SER-driver!',LF}
         MOVE.L   (A7)+,A0
IN_INIT  BSR      SENDINIT
         MOVEQ    #25,D3
         BSR      PAUSE
         MOVEQ    #IO.CLOSE,D0
         TRAP     #2                Close channel again
         MOVEQ    #25,D3
         BSR      PAUSE
         MOVEQ    #0,D0
INIT_END RTS

* Wait for phone call
                                    
SP_WTCAL MOVE.W   D3,D4             Save timeout
         BSR      OPENCHAN          Open channel
         BNE      INIT_END          Exit if error
         BSR      SENDINIT
         BSR      PURGE_IN
WTCALL_1 MOVE.W   D4,D3
         BSR      GETRESLT
         BNE.S    WTCAL_FL
         LEA      RINGMSG,A2
         BSR      CMPRESLT
         BNE.S    WTCALL_2
         MOVE.W   ANSWER,D1
         BNE.S    WTCALL_A
         PRLOG    {'PHONE IS RINGING',LF}
         BRA      WTCALL_1
WTCALL_A PRLOG    {'ANSWERING CALL',LF}
         LEA      ASWCMD,A1
         BSR      SENDCMD
         BRA      WTCALL_1
WTCALL_2 BSR      ACONNECT              ;<different entry for answer mode>
         CMPI.L   #FAXCALL,D0           ;<Check for fax connect attempted>
         BEQ.S    WTCAL_FF              ;<and here>
         TST.L    D0                    ;<to restore origional logic>
         BNE.S    WTCAL_F2
WTCAL_OK MOVEQ    #50,D3
         BSR      PAUSE
         MOVEQ    #0,D0
WTCAL_RT RTS
WTCAL_F2 BSR      MODEMLOG
WTCAL_FF                                ;<comes here after lfax>
         MOVEQ    #ERR.EF,D0        Failed to connect
WTCAL_FL MOVE.L   D0,-(A7)          Save error code
         MOVEQ    #IO.CLOSE,D0
         TRAP     #2                Close channel
         MOVE.L   (A7)+,D0
         RTS



* Disconnect line
                                    
SP_DSCON BSR      PURGE_IN
         LEA      HANGUP,A1
         TST.W    (A1)
         BEQ.S    DSCON_CL
         BSR      SENDCMD
         MOVE.W   #5*50,D3
DSCON_LP BSR      GETRESLT
         BNE.S    DSCON_CL
         LEA      OKMSG,A2
         BSR      CMPRESLT
         BNE      DSCON_LP
DSCON_CL MOVE.W   #100,D3
         BSR      PAUSE
         MOVEQ    #IO.CLOSE,D0
         TRAP     #2                Close channel (lowers SER3 DTR line)
         RTS

* Closedown
                                    
SP_CLOSE TST.B    D1                Online?
         BEQ.S    RESMOD            No, skip
         PRINT    {CR,LF,LF,'CLOSED - CALL BACK LATER',CR,LF}
         BSR      SP_DSCON
RESMOD   LEA      OFFLINE,A1
         TST.W    (A1)
         BEQ.S    SP_CLOS2
         BSR      SENDCMD
         MOVE.W   #5*50,D3
         BSR      GETRESLT
SP_CLOS2 MOVEQ    #IO.CLOSE,D0
         TRAP     #2                Close channel
         RTS

* Return possible originate modes

SP_FBAUD MOVE.W   BAUD,D2
         LEA      DIAL300,A0
         LEA      BAUDTBL,A1
         MOVEQ    #0,D0
         MOVEQ    #0,D1
FBAUD_LP CMP.W    (A1)+,D2
         BLO.S    FBAUD_EN
         TST.W    (A0)
         BEQ.S    FBAUD_NX
         BSET     D0,D1
FBAUD_NX ADDQ.W   #1,D0
         ADDA.W   #50,A0
         BRA      FBAUD_LP
FBAUD_EN MOVEQ    #0,D0
         RTS

* Originate call

SP_DIAL  BSR      PURGE_IN
         LEA      DIAL300,A3
         MOVE.W   D2,D0
         MULU     #50,D0
         ADDA.W   D0,A3             A3 = dial cmd
         LEA      BAUDTBL,A0
         ADD.W    D2,D2
         MOVE.W   (A0,D2.W),D2
         MOVE.W   BAUD,D0
         CMP.W    D0,D2
         BHI.S    SPD_BP            Baud rate too high
         TST.W    (A3)
         BEQ.S    SPD_BP            Error if no dial command
         MOVE.L   A1,A5             Number to dial
         BSR      OPENCHAN
         BNE.S    SPD_RTS
         MOVE.W   D2,D1
         BSR      SETBAUD
         MOVE.L   A3,A1
         BSR      SENDCMD
         MOVE.L   A5,A1
         BSR      SENDCMD
         LEA      DSUFFIX,A1
         BSR      SENDCMD
         MOVE.W   #70*50,D3
         BSR      GETRESLT
         BEQ.S    SPD_2
         PRLOG    {'MODEM TIMEOUT',LF}
         MOVEQ    #IO.SBYTE,D0
         MOVEQ    #CR,D1
         MOVEQ    #-1,D3
         TRAP     #3
         BRA.S    SPD_FL
SPD_2    LEA      BUSYMSG,A2
         BSR      CMPRESLT
         BEQ.S    SPD_BUSY
         BSR      CONNECT
         BEQ.S    SPD_OK
         BSR      MODEMLOG
         BRA.S    SPD_FL
SPD_OK   MOVEQ    #50,D3            Connect OK, wait 1 second
         BSR.S    PAUSE
         MOVEQ    #0,D0
         RTS
SPD_FL   MOVEQ    #ERR.NC,D0        No connect
         BRA.S    SPD_ERR
SPD_BUSY MOVEQ    #ERR.IU,D0        Busy
SPD_ERR  MOVE.L   D0,-(A7)
         MOVEQ    #IO.CLOSE,D0
         TRAP     #2
         MOVE.L   (A7)+,D0
         RTS
SPD_BP   MOVEQ    #ERR.BP,D0
SPD_RTS  RTS

* I/O routines
                                    
SP_PEND  MOVEQ    #IO.PEND,D0
         BRA.S    DOTRP3
SP_FBYTE MOVEQ    #IO.FBYTE,D0
         BRA.S    DOTRP3
SP_SBYTE MOVEQ    #IO.SBYTE,D0
         BRA.S    DOTRP3
SP_SSTRG MOVEQ    #IO.SSTRG,D0
         BRA.S    DOTRP3
SP_FLUSH MOVEQ    #FS.FLUSH,D0
         BRA.S    DOTRP3
SP_DUMP  MOVEQ    #IO.DUMP,D0
         BRA.S    DOTRP3
SP_PARIT MOVEQ    #IO.PARIT,D0
DOTRP3   TRAP     #3
         RTS

* Open channel

OPENCHAN MOVEQ    #IO.OPEN,D0
         MOVEQ    #0,D3
         LEA      PORT,A0
         TRAP     #2
         TST.L    D0
         RTS

* Set baudrate (D1)

SETBAUD  MOVE.W   LOCKED,D0
         BEQ.S    SB_2
         MOVE.W   BAUD,D1
SB_2     MOVEQ    #MT.BAUD,D0
         TRAP     #1                Use MT.BAUD AS MUST have HERMES
SB_END   MOVEQ    #25,D3

* Pause for D3 frames
                                    
PAUSE    MOVEM.L  D1/A0-A1,-(A7)
         MOVEQ    #MT.SUSJB,D0
         MOVEQ    #-1,D1
         SUBA.L   A1,A1
         TRAP     #1
         MOVEM.L  (A7)+,D1/A0-A1
         RTS

* Send init strings

SENDINIT MOVE.W   BAUD,D1
         BSR      SETBAUD
         MOVEQ    #2,D7
         LEA      INIT1,A3
DO_INIT1 MOVEQ    #2,D6
DO_INIT2 TST.W    (A3)
         BEQ.S    INIT_NXT
         MOVE.L   A3,A1
         BSR.S    SENDCMD
         MOVE.W   #5*50,D3
         BSR      GETRESLT
         BNE.S    IN_FAIL
         LEA      OKMSG,A2
         BSR      CMPRESLT
         BEQ.s    INIT_NXT
IN_FAIL  DBF      D6,DO_INIT2
         MOVE.L   A0,-(A7)
         SUBA.L   A0,A0
         PRINT    {'WARNING: Failed to initialise modem',LF}
         MOVE.L   (A7)+,A0
         RTS
INIT_NXT ADDA.W   #50,A3
         DBF      D7,DO_INIT1
         RTS

* Send modem command (A1)

SENDCMD  MOVE.W   (A1)+,D2
         SUBQ.W   #1,D2
         BLT.S    SC_END
SC_LOOP  MOVE.B   (A1)+,D1
         CMPI.B   #'|',D1
         BNE.S    SC_NOPIP
         MOVEQ    #CR,D1
SC_NOPIP CMPI.B   #'~',D1
         BNE.S    SC_SEND
         MOVEQ    #25,D3
         BRA.S    SC_PAUSE
SC_SEND  MOVE.L   A1,-(A7)
         MOVEQ    #IO.SBYTE,D0
         MOVEQ    #-1,D3
         TRAP     #3
         MOVE.L   (A7)+,A1
         MOVE.W   DELAY,D3
         BEQ.S    SC_NEXT
SC_PAUSE BSR      PAUSE
SC_NEXT  DBF      D2,SC_LOOP
SC_END   RTS

* Purge input buffer

PURGE_IN MOVEQ    #0,D3
         MOVEQ    #IO.FBYTE,D0
         TRAP     #3
         TST.L    D0
         BEQ      PURGE_IN
         RTS

* Get result from modem (D3: Timeout)

GETRESLT MOVEQ    #IO.FLINE,D0
         MOVEQ    #60,D2
         LEA      RESBUF,A1
         TRAP     #3
         CMPI.L   #ERR.NC,D0
         BEQ.S    GR_END
         CMPI.L   #ERR.NO,D0
         BEQ.S    GR_END
         CMPI.L   #ERR.EF,D0
         BEQ.S    GR_END
         TST.L    D0
         BNE      GETRESLT
         SUBA.W   D1,A1
         CMPI.W   #2,D1
         BLE      GETRESLT
GR_STRIP ANDI.B   #$7F,(A1)+
         SUBQ.W   #1,D1
         BGT      GR_STRIP
         MOVEQ    #0,D0
GR_END   TST.L    D0
         RTS

* Look for CONNECT codes

ACONNECT MOVE.B   #AMODE,D7         <entry flag>
         BRA.S    CONN
CONNECT  MOVE.B   #CMODE,D7         <entry flag>
CONN     MOVEQ    #ERR.EF,d0        <ensure origional logic + lable>
         MOVEQ    #CON_300,D1
         LEA      CONN300,A2        Start with CONNECT 300
CON_LOOP BSR      CMPRESLT          Jump if match found
         BEQ.S    CON_OK
         ADDA.W   #32,A2            Skip to next connect string
         ADDQ.W   #1,D1
         CMPI.W   #CON_NC,D1
         BLE      CON_LOOP          <keep going in loop until end>
         RTS
CON_OK   CMPI.W   #CON_FAX,D1       <fax connect ?>
         BEQ.S    DO_FAX            <yes do this>
         CMPI.W   #CON_NC,D1        <failed because is a FAX ?>
         BEQ.S    DO_DCNC           <do pickup>
         MOVE.W   D1,-(A7)          <here for data connect>
         ADD.W    D1,D1
         MOVE.W   BAUDTBL(PC,D1.W),D1
         BSR      SETBAUD           Set new baud rate
         MOVE.W   (A7)+,D1
         BSR      MODEMLOG          So report directly
         MOVE.W   #CON_9600,D1      Patch out higher rates
OK_RET   MOVEQ    #0,D0
         RTS

DO_FAX   BSR      MODEMLOG         <log this for QBOX>
         CMPI.B   #AMODE,D7        <is it answering?>
         BNE.S    NO_FAX
         PRLOG    {'Executing FAX Software',LF}
         jsr      confax           <call lfax>
endfax   neg.l    d0               <return code from fax>
         add.b    #'0',d0
         lea      FAXCODE,a1
         move.b   d0,-2(a1)
         lea      FAXMSG,a1
         jsr      (a4)             <log the return error code>
         move.l   #FAXCALL,D0      <return flag for answer routine>
         RTS                       <back to answer>

DO_DCNC  BSR      MODEMLOG         <log this for QBOX>
         CMPI.B   #AMODE,D7
         BNE.S    NO_FAX
         PRLOG    {'Executing FAX Software',LF}
         jsr      dcffax           <call lfax>
         bra.s    endfax

NO_FAX   move.l   #ERR.EF,D0       <don't fax if not answering>
         RTS

* The baud table is set for the next highest rate when no exact match
* is available

BAUDTBL  DC.W     300,1200,1200,2400,4800,9600,9600,19200,19200

* Compare result codes
* Exit: EQ if string in RESBUF matches string at (A2)
* Up to the first / if present.

CMPRESLT MOVEM.L  D0-D2/A1-A2,-(A7)
         MOVE.W   (A2)+,D2
         SUBQ.W   #1,D2
         BLT.S    CR_END
         LEA      RESBUF,A1
CR_LOOP  MOVE.B   (A1)+,D1
         BSR.S    TOUPPER
         MOVE.B   D1,D0
         MOVE.B   (A2)+,D1
         BSR.S    TOUPPER
         CMPI.B   #'|',D1
         BNE.S    CR_CMP
         MOVEQ    #CR,D1
CR_CMP   CMP.B    D1,D0
         DBNE     D2,CR_LOOP
CR_END   MOVEM.L  (A7)+,D0-D2/A1-A2
         RTS

TOUPPER  CMPI.B   #'a',D1
         BLO.S    TOUP_END
         CMPI.B   #'z',D1
         BHI.S    TOUP_END
         SUBI.B   #'a'-'A',D1
TOUP_END RTS

* Print modem message to logfile

MODEMLOG MOVE.B   #'"',-2(A1)
         MOVE.L   A1,D0
         LEA      REPORT,A1
         SUB.L    A1,D0
         MOVE.W   D0,-(A1)
         JMP      (A4)

         SECTION  MSG

         DC.W     0
REPORT   DC.B     'MODEM REPORTS "'
RESBUF   DS.B     60

         EXPAND
FAXMSG   STRING$  {'Fax Software Return Code -$?',LF}
FAXCODE
         DC.W     0

         END
